/*****************************************************************************\
|* 
|* C++ visualisation app for the IP space we have mapped to cities.
|* 
|* Author						: SjG
|* First created				: Sat Nov  8 13:55:51 GMT 2003
|* Modification history			: <none>
|* Tabstops						: 4
|* Licence						: GPL, See COPYING for details
|*
|*    This program is free software; you can redistribute it and/or modify
|*    it under the terms of the GNU General Public License as published by
|*    the Free Software Foundation; either version 2 of the License, or
|*    (at your option) any later version.
|*
|*    This program is distributed in the hope that it will be useful,
|*    but WITHOUT ANY WARRANTY; without even the implied warranty of
|*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
|*    GNU General Public License for more details.
|*
|*    You should have received a copy of the GNU General Public License
|*    along with this program; if not, write to the Free Software
|*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
|*
|*
|* 
|*           [3]----------[2]
|*           /|           /|
|*          / |          / |     
|*         /  |         /  |
|*        /   |        /   |
|*      [7]---+------[6]   |
|*       |   [0]------+---[1]
|*       |   /        |   /
|*       |  /         |  /
|*       | /          | /
|*       |/           |/
|*      [4]----------[5]
|*
\*****************************************************************************/
#include "cube.h"
#include <GL/glu.h>
/*****************************************************************************\
|* Constructor
\*****************************************************************************/
Cube::Cube(float nx, float ny, float nz, float sz,
                GLubyte r, GLubyte g, GLubyte b)
                : ox(2*(nx+sz/2)-1), oy(2*(ny+sz/2)-1), oz(2*(nz+sz/2)-1),
                size(sz)
{
        x[0] = x[3] = x[4] = x[7] = (2 * nx) - 1.0f;
        x[1] = x[2] = x[5] = x[6] = (2 * (nx + sz)) - 1.0f;

        y[0] = y[1] = y[4] = y[5] = (2 * ny) - 1.0f;
        y[2] = y[3] = y[6] = y[7] = (2 * (ny + sz)) - 1.0f;

        z[0] = z[1] = z[2] = z[3] = (2 * nz) - 1.0f;
        z[4] = z[5] = z[6] = z[7] = (2 * (nz + sz)) - 1.0f;

        colour[0] = r;
        colour[1] = g;
        colour[2] = b;
}


void Cube::draw() const         // Daniel K. O.
{
        glColor3ubv(colour);
        glBegin(GL_POINTS);
                 glVertex3f(ox, oy, oz);
        glEnd();
}

void Cube::draw2() const         // Daniel K. O.
{
        glColor3ubv(colour);
        glBegin(GL_QUADS);

                glVertex3f(x[0], y[0], z[0]); // front
                glVertex3f(x[3], y[3], z[3]);
                glVertex3f(x[2], y[2], z[2]);
                glVertex3f(x[1], y[1], z[1]);

                glVertex3f(x[4], y[4], z[4]); // back
                glVertex3f(x[5], y[5], z[5]);
                glVertex3f(x[6], y[6], z[6]);
                glVertex3f(x[7], y[7], z[7]);

                glVertex3f(x[0], y[0], z[0]); // bottom
                glVertex3f(x[1], y[1], z[1]);
                glVertex3f(x[5], y[5], z[5]);
                glVertex3f(x[4], y[4], z[4]);

                glVertex3f(x[3], y[3], z[3]); // top
                glVertex3f(x[7], y[7], z[7]);
                glVertex3f(x[6], y[6], z[6]);
                glVertex3f(x[2], y[2], z[2]);

                glVertex3f(x[0], y[0], z[0]); // left
                glVertex3f(x[4], y[4], z[4]);
                glVertex3f(x[7], y[7], z[7]);
                glVertex3f(x[3], y[3], z[3]);

                glVertex3f(x[1], y[1], z[1]); // right
                glVertex3f(x[2], y[2], z[2]);
                glVertex3f(x[6], y[6], z[6]);
                glVertex3f(x[5], y[5], z[5]);
        glEnd();
}

void Cube::draw3() const
{
        GLUquadric      *circ;
        
        circ = gluNewQuadric();
        gluQuadricDrawStyle(circ, GLU_LINE);
        
        glColor3ubv(colour);
        glPushMatrix();
        glTranslated(ox, oy, oz);
        gluSphere(circ, size/2, 16, 16);
        glPopMatrix();
        
        gluDeleteQuadric(circ);
}


