/*****************************************************************************\
|* 
|* C++ visualisation app for the IP space we have mapped to cities.
|* 
|* Author						: SjG
|* First created				: Sat Nov  8 13:55:51 GMT 2003
|* Modification history			: <none>
|* Tabstops						: 4
|* Licence						: GPL, See COPYING for details
|*
|*    This program is free software; you can redistribute it and/or modify
|*    it under the terms of the GNU General Public License as published by
|*    the Free Software Foundation; either version 2 of the License, or
|*    (at your option) any later version.
|*
|*    This program is distributed in the hope that it will be useful,
|*    but WITHOUT ANY WARRANTY; without even the implied warranty of
|*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
|*    GNU General Public License for more details.
|*
|*    You should have received a copy of the GNU General Public License
|*    along with this program; if not, write to the Free Software
|*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
|*
\*****************************************************************************/

#include <cstdio>
#include <cstdlib>
#include <list>       // Daniel K. O.
#include <cstring>      // Daniel K. O.
#include <cmath>        // Daniel K. O.

//#include <glib.h>     Daniel K. O.

#include "SDL.h" 
#include "SDL_net.h" 
#include <GL/gl.h>
#include <GL/glu.h>

#include "cmdLine.h"
#include "cube.h"
#include "cubeFetcher.h"

void unlockSurface(SDL_Surface *screen);
void lockSurface(SDL_Surface *screen);

using namespace std;


static void reshape(int w, int h)
{
        glViewport(0, 0, w, h);
        glMatrixMode(GL_PROJECTION);
        glLoadIdentity();
        gluPerspective(40, (float)w/(float)h, 0.001, 30);
        glMatrixMode(GL_MODELVIEW);
}

static void setdist(float d)
{
        glLoadIdentity();
        glTranslatef(0, 0, d);
}


int main(int argc, char **argv) {
        /*************************************************************************\
        |* Get the rotation parameters
        \*************************************************************************/
        cmdlineArgs *args = new cmdlineArgs(argc, argv);
        float x	        = args->getFloat("-rx", "--rotate-x", 10);
        float y	        = args->getFloat("-ry", "--rotate-y", 5);
        float z         = args->getFloat("-rz", "--rotate-z", 5);
        float rX        = (float) (x / 180.0 * M_PI) / 20;
        float rY        = (float) (y / 180.0 * M_PI) / 20;
        float rZ        = (float) (z / 180.0 * M_PI) / 20;

        /*************************************************************************\
        |* Get the foreground and background colours
        \*************************************************************************/
        Uint32 bgR,bgG,bgB,fgR,fgG,fgB;
        char *bgstr	= args->getString("-bg", "--background-colour", "0,0,0");
        char *fgstr	= args->getString("-fg", "--foreground-colour", "255,255,255");
        sscanf(bgstr, "%d,%d,%d", &bgR, &bgG, &bgB);
        sscanf(fgstr, "%d,%d,%d", &fgR, &fgG, &fgB);

        /*************************************************************************\
        |* Server URL data
        \*************************************************************************/
        char *url       = args->getString("-u", "--url", "http://hostip.info/vis/countries.html");

        /*************************************************************************\
        |* Optional highlight an ip box
        \*************************************************************************/
        char *ip        = args->getString("-ip", "--ip", "81.99.225.13");

        /*************************************************************************\
        |* Get the display width and height
        \*************************************************************************/
        int W	= args->getInt("-w", "--width", 500);
        int H	= args->getInt("-h", "--height", 500);

        /*************************************************************************\
        |* Initialise SDL and SDLnet
        \*************************************************************************/
        if ((SDL_Init(SDL_INIT_VIDEO|SDL_INIT_TIMER))) {
                printf("Could not initialize SDL: %s.\n", SDL_GetError());
                exit(-1);
        }
        if (SDLNet_Init()) {
                printf("SDLNet_Init: %s\n", SDLNet_GetError());
                exit(2);
        }
        atexit(SDL_Quit);

        /*************************************************************************\
        |* Open up a window for output
        \*************************************************************************/
        SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 1 ); // Daniel K. O.
        SDL_Surface *screen = SDL_SetVideoMode(W, H, 0, SDL_OPENGL | SDL_RESIZABLE);
        if ( !screen ) {
                fprintf(stderr, "Couldn't set %dx%dx16 video mode: %s\n", W, H,
                        SDL_GetError());
                exit(1);
    	}

        /*************************************************************************\
        |* Create the Cube container and add a bounding (0,0,0)->(1,1,1) cube
        \*************************************************************************/
        list<Cube>      cubes;
        list<Cube>      highcubes;
        Cube maincube(0,0,0,1, fgR, fgG, fgB);

        /*************************************************************************\
        |* Add in the IP to highlight
        \*************************************************************************/
        int aa,bb,cc;
        if ((sscanf(ip, "%d.%d.%d", &aa, &bb, &cc)) == 3) {
                float fx = ((aa-2) * 1.0f) / 256.0f;
                float fy = ((bb-2) * 1.0f) / 256.0f;
                float fz = ((cc-2) * 1.0f) / 256.0f;
                float fs = (5.0f/256.0f);

                highcubes.push_back( Cube(fx, fy, fz, fs, 255, 255, 255) );
        }

        /*************************************************************************\
        |* Fetch the data from the server
        \*************************************************************************/
        CubeFetcher::fetch(cubes, url);

        char buf[255]; // Daniel K. O.
        snprintf(buf, sizeof(buf), "Internet Map - %d IPs - http://hostip.info", cubes.size()); // Daniel K. O.
        SDL_WM_SetCaption(buf, NULL); // Daniel K. O.

        /*************************************************************************\
        |* Run the event loop
        \*************************************************************************/
        bool done = false;



        // OpenGL Setup, by Daniel K. O.
        glClearColor(bgR/255., bgG/255., bgB/255., 0);
        glEnable(GL_DEPTH_TEST);
        glDepthFunc(GL_LEQUAL);
//        glEnable(GL_BLEND);
//        glBlendFunc(GL_SRC_ALPHA, GL_SRC_ALPHA);
        glShadeModel(GL_FLAT);
        glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
//        glEnable(GL_CULL_FACE);
//        glEnable(GL_POINT_SMOOTH);
        glPointSize(2);

        // reshape, by Daniel K. O.
        reshape(W, H);

        float dist = -3;
        setdist(dist);


        float spinx=0, spiny=0;        // Daniel K. O.

        bool list_done = false;

        SDL_Event event;
        Uint8 *keys;
        
        
        // creates the display-list
        glNewList(1, GL_COMPILE);
                maincube.draw2();
                for (list<Cube>::const_iterator i=highcubes.begin(); i!=highcubes.end(); ++i)
                        i->draw3();
                for (list<Cube>::const_iterator i=cubes.begin(); i!=cubes.end(); ++i)
                        i->draw();
        glEndList();
        highcubes.clear();
        cubes.clear();
        
        
        while (! done ) {
                while (SDL_PollEvent(&event)) {

                        switch (event.type) {
                                case SDL_MOUSEMOTION:
                                        if (SDL_GetMouseState(NULL, NULL)) {
                                                rX += event.motion.yrel/20.;
                                                rY += event.motion.xrel/20.;
                                        }
					break;

                                case SDL_KEYDOWN:
                                        switch (event.key.keysym.sym) {
                                                case SDLK_q:
                                                case SDLK_ESCAPE:
                                                        done = true;
                                                        break;
                                                case SDLK_SPACE:
                                                        rX = rY = 0;
                                                        break;
                                        }
                                        break;
                                case SDL_VIDEORESIZE:
                                        reshape(event.resize.w, event.resize.h);
                                        break;
                                case SDL_QUIT:
                                        done = true;
                                        break;
                        } /* switch */
                }
                spinx += rX;
                spiny += rY;
                while (spinx>360)
                        spinx-=360;
                while (spinx<0)
                        spinx+=360;
                while (spiny>360)
                        spiny-=360;
                while (spiny<0)
                        spiny+=360;

                keys = SDL_GetKeyState(NULL);
                if (keys[SDLK_KP_PLUS])         setdist(dist+=0.01);
                if (keys[SDLK_KP_MINUS])        setdist(dist-=0.01);


                glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT ); // Daniel K. O.

                glPushMatrix();
                glRotatef(spinx, 1, 0, 0);
                glRotatef(spiny, 0, 1, 0);
                glRotatef(rZ, 0, 0, 1);

                glCallList(1);

                glPopMatrix();

                SDL_GL_SwapBuffers();

        }

        /*************************************************************************\
        |* Shut down everything cleanly
        \*************************************************************************/
        SDLNet_Quit();
        SDL_Quit();
        return 0;
}


